/*
 * file:       Base.c
 * purpose:    basic functions like common initialization, common interrupts, ...
 * project:    AVR-Timer
 * author:     Santiago M. Hernandez
 * copyright:  2008 (c) Creative commons - by-nc-sa - some rights reserved
 *             see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "Config.h"
#include "Base.h"
#include "SPI.h"
#ifdef HAVE_LCD
#include "LCD.h"
#endif


void init(void) {
#ifdef WANT_WHISTLE   
   DDR_BUZZ         |= 1 << PIN_BUZZ;
#endif
   DDR_UV_SWITCH    |= 1 << PIN_UV_SWITCH;
   DDR_LIGHT_SWITCH |= 1 << PIN_LIGHT_SWITCH;

   /*
    * timer 1 - time base
    */
   TCCR1A = 1 << WGM11 | 1 << WGM10;                       // fast PWM mode
   TCCR1B = 1 << WGM13 | 1 << WGM12                        // with OCR1A as top
          | 1 << CS10                                      // prescale: 1 == sysclock 
#ifdef TEST_MODE          
          | 1 << CS12                                      // prescale: 1024
#endif   
            ;
   OCR1A  = TIMER_TOP;

#ifdef __AVR_ATtiny2313__
   TIMSK |= 1 << TOIE1                                     // no PWM on attiny
#endif
#ifdef __AVR_ATmega88__ 
   TIMSK1 = 1 << TOIE1                    
#endif   
#ifdef __AVR_ATmega16__ 
   TIMSK |= 1 << TOIE1 
#endif
          ;

#ifdef WANT_WHISTLE   
   initBeeper();
#endif   

#ifdef HAVE_LCD
   lcdInit();
#else   
   spiInit();
#endif

   sei();                                                  // allow interrupts
}


/*
 * overflow of timer 1 is time base
 */
ISR(TIMER1_OVF_vect) {
   /*
    * time base
    */
   if (++status.cIPS >= TIMER_IPS) {
      status.cIPS = 0;
      status.time++;
   }

   /*
    * push buttons
    */
   if (!(status.cIPS % IO_CHECK)) {
      if (PORT_T1 & 1 << PIN_T1) status.btT1++;
      if (PORT_T2 & 1 << PIN_T2) status.btT2++;
      if (PORT_T3 & 1 << PIN_T3) status.btT3++;
      if (PORT_T4 & 1 << PIN_T4) status.btT4++;
   }
}


#ifdef WANT_WHISTLE   
ISR(BEEPER_ISR_vect) {
   if (status.beep) {
      PORT_BUZZ ^= 1 << PIN_BUZZ;
      if (!--status.beep) {
         setBeeperFrequency(WHISTLE);
      }
   }
}
#endif   


/*
 * make noise in case of unintended interrupt call
 */
ISR(BADISR_vect) {
#ifdef WANT_WHISTLE   
   setBeeperFrequency(WHISTLE >> 2);                         // ca. 1760 Hz
   status.beep = 1500;
#endif   
}

