#ifndef BASE_H
#define BASE_H
/*
 * file:       Base.h
 * purpose:    basic declarations
 * project:    AVR-Timer
 * author:     Santiago M. Hernandez
 * copyright:  2008 (c) Creative commons - by-nc-sa - some rights reserved
 *             see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#include <inttypes.h>
#include "Config.h"
#ifdef EXTERN
#  undef EXTERN
#endif
#ifdef PLACE_GLOBALS_HERE
#  define EXTERN 
#else
#  define EXTERN extern
#endif
extern void init(void);

#define COUNTER_MAX  1800

enum WorkMode {
   MD_CONFIG,
   MD_LIGHT,
   MD_COUNTDOWN,
   MD_DONE
};

struct Work   {
   struct Display {
      uint32_t step0;
      uint32_t step1;
   }                 display;
   int16_t           counter;
   int16_t           countDown;
   uint8_t           mode;
};

struct Status {
   volatile uint16_t cIPS;
#ifdef WANT_WHISTLE   
   volatile uint16_t beep;
#endif   
   volatile uint32_t time;
   volatile uint32_t otime;
#ifdef HAVE_LCD   
   volatile uint8_t  lcdPos;
#endif   
   volatile uint8_t  btT1;
   volatile uint8_t  btT2;
   volatile uint8_t  btT3;
   volatile uint8_t  btT4;
   struct Work       work;
};

EXTERN volatile struct Status status
#ifdef PLACE_GLOBALS_HERE
#  ifdef WANT_WHISTLE
= { .beep = 400,                                           // signal initialization
    .work = { .counter = 330 }}
#  endif
#endif
;


/*
 * timer calculations
 */
#ifdef __AVR_ATtiny2313__
#  define F_PWM              2000
#  define TIMER_TOP          (F_CPU / F_PWM - 1)
#endif
#ifdef __AVR_ATmega88__
#  define F_PWM              20000
#  define TIMER_TOP          (F_CPU / F_PWM - 1)
#endif
#ifdef __AVR_ATmega16__
#  define F_PWM              20000
#  define TIMER_TOP          (F_CPU / F_PWM - 1)
#endif

#define WHISTLE              (((F_CPU / 100000) * 440) / 1000)
#define TIMER_IPS            F_PWM

#ifdef __AVR_ATtiny2313__
#  define ENC_CHECK          2
#  define IO_CHECK           40
#else
#  define ENC_CHECK          20
#  define IO_CHECK           400
#endif

#define BUTTON_SAMPLING      10                            // check button that times
#define USER_TIMEOUT         10                            // timeout in seconds
#ifdef WANT_WHISTLE   
#  ifdef __AVR_ATtiny2313__ 
#    define BEEPER_ISR_vect  TIMER0_OVF_vect 
#  endif
#  ifdef __AVR_ATmega88__
#    define BEEPER_ISR_vect  TIMER2_OVF_vect
#  endif
#  ifdef __AVR_ATmega16__
#    define BEEPER_ISR_vect  TIMER2_COMP_vect
#  endif
static __inline__ void setBeeperFrequency(uint8_t f) {
#  ifdef __AVR_ATtiny2313__ 
   OCR0A = f;
#  endif
#  ifdef __AVR_ATmega88__ 
   OCR2A = f;
#  endif
#  ifdef __AVR_ATmega16__ 
   OCR2  = f;
#  endif
}
static __inline__ void initBeeper(void) {
#  ifdef __AVR_ATtiny2313__
   TCCR0A = 1 << WGM01 | 1 << WGM00;                       // using fast PWM mode with CTC
   TCCR0B = 1 << WGM02                                     // ensures synchronized update of top level 
          | 1 << CS02;                                     // prescale of 256 (245 Hz - 20 kHz)
   TIMSK |= 1 << TOIE0;                                    // enable interrupt
#  endif   
#  ifdef __AVR_ATmega88__ 
   TCCR2A = 1 << WGM21 | 1 << WGM20;                       // using fast PWM mode with CTC
   TCCR2B = 1 << WGM22                                     // ensures synchronized update of top level 
          | 1 << CS22 | 1 << CS21;                         // prescale of 256 (245 Hz - 20 kHz)
   TIMSK2 = 1 << TOIE2;                                    // enable interrupts
#  endif
#  ifdef __AVR_ATmega16__ 
   TCCR2  = 1 << WGM21                                     // CTC mode (has no fast PWM mode with CTC)
          | 1 << CS22 | 1 << CS21;                         // prescale of 256 (245 Hz - 20 kHz)
   TIMSK |= 1 << OCIE2;                                    // enable interrupts
#  endif   
   setBeeperFrequency(WHISTLE);
}
#endif
#endif
