/*
 * file:       SPI.c
 * purpose:    serial SPI communication
 * project:    common
 * author:     Santiago M. Hernandez
 * copyright:  2008 (c) Creative commons - by-nc-sa - some rights reserved
 *             see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#include <stdint.h>
#include <avr/io.h>
#include "Config.h"
#include "SPI.h"

void spiInit(void) {
   DDR_COMM_CLOCK |= 1 << PIN_COMM_CLOCK;
   DDR_COMM_DATA  |= 1 << PIN_COMM_DATA;
   DDR_COMM_SYNC  |= 1 << PIN_COMM_SYNC;
}

static void spiSync(void) {
   PORT_COMM_SYNC ^= 1 << PIN_COMM_SYNC;
   asm volatile ("nop");
   PORT_COMM_SYNC ^= 1 << PIN_COMM_SYNC;
}


void spiWrite(uint8_t *p, uint8_t s, enum SyncMode sm) {
   uint8_t n=0;

   do {
      spiSendByte(p[n]);
      if (sm == SYNC_EACH) spiSync();
   } while (++n < s);
   if (sm == SYNC_ONCE) spiSync();
}


void spiSendByte(uint8_t b) {      
      COMM_DATA = b;

#if defined(__AVR_ATmega88__) || defined(__AVR_ATmega16__)     
      while (!(SPSR & (1 << SPIF)));
#endif
#ifdef __AVR_ATtiny2313__
      USICR = COMM_CLOCK_LOW;
      USICR = COMM_CLOCK_HIGH;
      USICR = COMM_CLOCK_LOW;
      USICR = COMM_CLOCK_HIGH;
      USICR = COMM_CLOCK_LOW;
      USICR = COMM_CLOCK_HIGH;
      USICR = COMM_CLOCK_LOW;
      USICR = COMM_CLOCK_HIGH;
      USICR = COMM_CLOCK_LOW;
      USICR = COMM_CLOCK_HIGH;
      USICR = COMM_CLOCK_LOW;
      USICR = COMM_CLOCK_HIGH;
      USICR = COMM_CLOCK_LOW;
      USICR = COMM_CLOCK_HIGH;
      USICR = COMM_CLOCK_LOW;
      USICR = COMM_CLOCK_HIGH;
#endif      
}
