/*
 * file:       SegDisp.c
 * purpose:    functions to handle pollin led-segment display (D120150)
 * project:    AVR-Timer
 * author:     Santiago M. Hernandez
 * copyright:  2008 (c) Creative commons - by-nc-sa - some rights reserved
 *             see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#include <inttypes.h>
#include <avr/io.h>
#include "Config.h"
#include "Base.h"
#include "SegDisp.h"


void sdGenDisplay(void) {
   uint8_t  h, m;
   uint8_t  h0, h1, m0, m1;
   uint32_t d0 = GND1;
   uint32_t d1 = GND2;

   switch (status.work.mode) {
   case MD_LIGHT:
      h0 = 10;
      h1 = 10;
      m0 = 10;
      m1 = 10;
      break;

   case MD_COUNTDOWN:
      h = status.work.countDown / 60;
      m = status.work.countDown - (h * 60);
      if (status.work.countDown & 0x01)
         d0 |= DCC;
      h0 = h / 10;
      h1 = h % 10;
      m0 = m / 10;
      m1 = m % 10;
      break;

   case MD_DONE:
      h0 = 11;
      h1 = 11;
      m0 = 11;
      m1 = 11;
      break;

   default:
      h = status.work.counter / 60;
      m = status.work.counter - (h * 60);
      d0 |= DCC;
      h0 = h / 10;
      h1 = h % 10;
      m0 = m / 10;
      m1 = m % 10;
      break;
   }
   switch (h0) {
   case 0:  d0 |= S1D1_0; d1 |= S2D1_0; break;
   case 1:  d0 |= S1D1_1; d1 |= S2D1_1; break;
   case 2:  d0 |= S1D1_2; d1 |= S2D1_2; break;
   case 3:  d0 |= S1D1_3; d1 |= S2D1_3; break;
   case 4:  d0 |= S1D1_4; d1 |= S2D1_4; break;
   case 5:  d0 |= S1D1_5; d1 |= S2D1_5; break;
   case 6:  d0 |= S1D1_6; d1 |= S2D1_6; break;
   case 7:  d0 |= S1D1_7; d1 |= S2D1_7; break;
   case 8:  d0 |= S1D1_8; d1 |= S2D1_8; break;
   case 9:  d0 |= S1D1_9; d1 |= S2D1_9; break;
   case 10: d0 |= S1D1_Y; d1 |= S2D1_Y; break;
   default: d0 |= S1D1_X; d1 |= S2D1_X; break;
   }

   switch (h1) {
   case 0:  d0 |= S1D2_0; d1 |= S2D2_0; break;
   case 1:  d0 |= S1D2_1; d1 |= S2D2_1; break;
   case 2:  d0 |= S1D2_2; d1 |= S2D2_2; break;
   case 3:  d0 |= S1D2_3; d1 |= S2D2_3; break;
   case 4:  d0 |= S1D2_4; d1 |= S2D2_4; break;
   case 5:  d0 |= S1D2_5; d1 |= S2D2_5; break;
   case 6:  d0 |= S1D2_6; d1 |= S2D2_6; break;
   case 7:  d0 |= S1D2_7; d1 |= S2D2_7; break;
   case 8:  d0 |= S1D2_8; d1 |= S2D2_8; break;
   case 9:  d0 |= S1D2_9; d1 |= S2D2_9; break;
   case 10: d0 |= S1D2_Y; d1 |= S2D2_Y; break;
   default: d0 |= S1D2_X; d1 |= S2D2_X; break;
   }

   switch (m0) {
   case 0:  d0 |= S1D3_0; d1 |= S2D3_0; break;
   case 1:  d0 |= S1D3_1; d1 |= S2D3_1; break;
   case 2:  d0 |= S1D3_2; d1 |= S2D3_2; break;
   case 3:  d0 |= S1D3_3; d1 |= S2D3_3; break;
   case 4:  d0 |= S1D3_4; d1 |= S2D3_4; break;
   case 5:  d0 |= S1D3_5; d1 |= S2D3_5; break;
   case 6:  d0 |= S1D3_6; d1 |= S2D3_6; break;
   case 7:  d0 |= S1D3_7; d1 |= S2D3_7; break;
   case 8:  d0 |= S1D3_8; d1 |= S2D3_8; break;
   case 9:  d0 |= S1D3_9; d1 |= S2D3_9; break;
   case 10: d0 |= S1D3_Y; d1 |= S2D3_Y; break;
   default: d0 |= S1D3_X; d1 |= S2D3_X; break;
   }

   switch (m1) {
   case 0:  d0 |= S1D4_0; d1 |= S2D4_0; break;
   case 1:  d0 |= S1D4_1; d1 |= S2D4_1; break;
   case 2:  d0 |= S1D4_2; d1 |= S2D4_2; break;
   case 3:  d0 |= S1D4_3; d1 |= S2D4_3; break;
   case 4:  d0 |= S1D4_4; d1 |= S2D4_4; break;
   case 5:  d0 |= S1D4_5; d1 |= S2D4_5; break;
   case 6:  d0 |= S1D4_6; d1 |= S2D4_6; break;
   case 7:  d0 |= S1D4_7; d1 |= S2D4_7; break;
   case 8:  d0 |= S1D4_8; d1 |= S2D4_8; break;
   case 9:  d0 |= S1D4_9; d1 |= S2D4_9; break;
   case 10: d0 |= S1D4_Y; d1 |= S2D4_Y; break;
   default: d0 |= S1D4_X; d1 |= S2D4_X; break;
   }
   status.work.display.step0 = d0;
   status.work.display.step1 = d1;
}





