#ifndef SEGDISP_H
#define SEGDISP_H
/*
 * file:       SegDisp.h
 * purpose:    declare constants for handling of pollin led-segment display (D120150)
 * project:    AVR-Timer
 * author:     Santiago M. Hernandez
 * copyright:  2008 (c) Creative commons - by-nc-sa - some rights reserved
 *             see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#include <inttypes.h>

#define SR1    0
#define SR2    8
#define SR3    16

#define BIT_A  0x01l
#define BIT_B  0x02l
#define BIT_C  0x04l
#define BIT_D  0x08l
#define BIT_E  0x10l
#define BIT_F  0x20l
#define BIT_G  0x40l
#define BIT_H  0x80l

// GND
#define GND1   (BIT_H << SR3 | BIT_H << SR1)
#define GND2   (BIT_C << SR3 | BIT_A << SR1)

// dots
#define DAM    (BIT_D << SR3)
#define DPM    (BIT_E << SR3)
#define DCC    (BIT_G << SR1)
#define DAL    (BIT_E << SR1)

// segments digit 1
#define D1A    (BIT_G << SR3)
#define D1B    (BIT_F << SR3)
#define D1C    (BIT_D << SR2)
#define D1D    (BIT_A << SR3)
#define D1E    (BIT_A << SR3)
#define D1F    0
#define D1G    (BIT_G << SR3)

// segments digit 2
#define D2A    (BIT_E << SR2)
#define D2B    (BIT_C << SR2)
#define D2C    (BIT_B << SR2)
#define D2D    (BIT_B << SR2)
#define D2E    (BIT_D << SR2)
#define D2F    (BIT_E << SR2)
#define D2G    (BIT_C << SR2)

// segments digit 3
#define D3A    (BIT_F << SR2)
#define D3B    (BIT_G << SR2)
#define D3C    (BIT_H << SR2)
#define D3D    (BIT_H << SR2)
#define D3E    (BIT_A << SR2)
#define D3F    (BIT_F << SR2)
#define D3G    (BIT_G << SR2)

// segments digit 4
#define D4A    (BIT_B << SR1)
#define D4B    (BIT_D << SR1)
#define D4C    (BIT_C << SR1)
#define D4D    (BIT_C << SR1)
#define D4E    (BIT_A << SR2)
#define D4F    (BIT_B << SR1)
#define D4G    (BIT_D << SR1)

// step 1 digit definitions
#define S1D1_1 0
#define S1D2_1 0
#define S1D3_1 (D3B | D3C)
#define S1D4_1 0

#define S1D1_2 (D1A | D1D)
#define S1D2_2 (D2D | D2E | D2G)
#define S1D3_2 (D3A | D3B)
#define S1D4_2 (D4A | D4B)

#define S1D1_3 (D1A | D1D)
#define S1D2_3 (D2D | D2G)
#define S1D3_3 (D3A | D3B | D3C)
#define S1D4_3 (D4D | D4G)

#define S1D1_4 0
#define S1D2_4 (D2F | D2G)
#define S1D3_4 (D3B | D3C)
#define S1D4_4 (D4F | D4G)

#define S1D1_5 0
#define S1D2_5 (D2D | D2F | D2G)
#define S1D3_5 (D3A | D3C)
#define S1D4_5 (D4D | D4F | D4G)

#define S1D1_6 0
#define S1D2_6 (D2D | D2E | D2F | D2G)
#define S1D3_6 (D3A | D3C)
#define S1D4_6 (D4D | D4E | D4F | D4G)

#define S1D1_7 0
#define S1D2_7 0
#define S1D3_7 (D3A | D3B | D3C)
#define S1D4_7 0              

#define S1D1_8 0
#define S1D2_8 (D2D | D2E | D2F | D2G)
#define S1D3_8 (D3A | D3B | D3C)
#define S1D4_8 (D4D | D4E | D4F | D4G)

#define S1D1_9 0
#define S1D2_9 (D2D | D2F | D2G)
#define S1D3_9 (D3A | D3B | D3C)
#define S1D4_9 (D4D | D4F | D4F)

#define S1D1_0 0
#define S1D2_0 (D2D | D2F)
#define S1D3_0 (D3A | D3B | D3C)
#define S1D4_0 (D4D | D4F)

#define S1D1_X (D1A | D1D)
#define S1D2_X (D2D | D2G)
#define S1D3_X (D3A)              
#define S1D4_X (D4D | D4G)

#define S1D1_Y 0
#define S1D2_Y (D2E | D2F)
#define S1D3_Y (D3B | D3C)        
#define S1D4_Y (D4E | D4F)

// step 2 digit definitions
#define S2D1_1 (D1B | D1C)
#define S2D2_1 (D2B | D2C)
#define S2D3_1 0            
#define S2D4_1 (D4B | D4C)

#define S2D1_2 (D1B | D1E)
#define S2D2_2 (D2A | D2B)
#define S2D3_2 (D3D | D3E | D3G)
#define S2D4_2 (D4D | D4E | D4G)

#define S2D1_3 (D1B | D1C)
#define S2D2_3 (D2A | D2B | D2C)
#define S2D3_3 (D3D | D3G)
#define S2D4_3 (D4A | D4B | D4C)

#define S2D1_4 0
#define S2D2_4 (D2B | D2C)
#define S2D3_4 (D3F | D3G)
#define S2D4_4 (D4B | D4C)

#define S2D1_5 0
#define S2D2_5 (D2A | D2C)
#define S2D3_5 (D3D | D3F | D3G)
#define S2D4_5 (D4A | D4C)

#define S2D1_6 0
#define S2D2_6 (D2A | D2C)
#define S2D3_6 (D3D | D3E | D3F | D3G)
#define S2D4_6 (D4A | D4C)

#define S2D1_7 0
#define S2D2_7 (D2A | D2B | D2C)
#define S2D3_7 0             
#define S2D4_7 (D4A | D4B | D4C)

#define S2D1_8 0
#define S2D2_8 (D2A | D2B | D2C)
#define S2D3_8 (D3D | D3E | D3F | D3G)
#define S2D4_8 (D4A | D4B | D4C)

#define S2D1_9 0
#define S2D2_9 (D2A | D2B | D2C)
#define S2D3_9 (D3D | D3E | D3F | D3G)
#define S2D4_9 (D4A | D4B | D4C)

#define S2D1_0 0
#define S2D2_0 (D2A | D2B | D2C)
#define S2D3_0 (D3D | D3E | D3F)
#define S2D4_0 (D4A | D4B | D4C)

#define S2D1_X (D1G)
#define S2D2_X (D2A)               
#define S2D3_X (D3D | D3G)           
#define S2D4_X (D4A)              

#define S2D1_Y (D1B | D1C)
#define S2D2_Y (D2B | D2C)         
#define S2D3_Y (D3E | D3F)           
#define S2D4_Y (D4B | D4C)        

extern void sdGenDisplay(void);
#endif
