/*
 * file:       main.c
 * purpose:    second level of operation (low priority functions)
 * project:    AVR-Timer
 * author:     Santiago M. Hernandez
 * copyright:  2008 (c) Creative commons - by-nc-sa - some rights reserved
 *             see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#include <inttypes.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#define PLACE_GLOBALS_HERE
#include "Config.h"
#include "Base.h"
#include "SegDisp.h"
#include "SPI.h"
#ifdef HAVE_LCD
#include "LCD.h"
#ifdef WANT_CLOCK
#include "clock.h"
#endif

PROGMEM char line1[] = "Mode: ";
PROGMEM char line2[] = "Time: ";

static void lcdDisplay(void) {
   char buf[12];

   lcdPrintLine(1, line1);
   switch (status.work.mode) {
   case MD_CONFIG:    lcdPrint("Config");    break;
   case MD_LIGHT:     lcdPrint("Light");     break;
   case MD_COUNTDOWN: lcdPrint("Countdown"); break;
   case MD_DONE:      lcdPrint("Done");      break;
   default:           lcdPrint("unknown");   break;                      
   }
   displayTime();
   lcdEndLine();

   lcdPrintLine(2, line2);
   switch (status.work.mode) {
   case MD_CONFIG:
      itoa(status.work.counter / 60, buf, 10); 
      lcdPrint(buf);
      lcdPrint(":");
      itoa(status.work.counter % 60, buf, 10);
      lcdPrint(buf);
      break;

   case MD_LIGHT:
      lcdPrint("L I G H T");
      break;

   case MD_COUNTDOWN:   
      itoa(status.work.countDown / 60, buf, 10); 
      lcdPrint(buf);
      lcdPrint(":");
      itoa(status.work.countDown % 60, buf, 10);
      lcdPrint(buf);
      break;

   case MD_DONE:   
      lcdPrint("D O N E");
      break;
   }
/*   
   lcdPrint(" ");
   itoa(TIMER_TOP, buf, 10);
   lcdPrint(buf);
   lcdPrint(" ");
   itoa(TIMER_IPS, buf, 10);
   lcdPrint(buf);
 */   
   lcdEndLine();
}
#endif


void main(void) {
#ifndef HAVE_LCD
   uint8_t  step = 0;
#endif   
   uint8_t  changed = 1;


   init();

   for (;; changed=0) {
      /*
       * button 1 is "start countdown"
       */
      if (status.btT1 > BUTTON_SAMPLING) {
         status.btT1 = 0;
         if (status.work.mode == MD_DONE) {
            status.work.mode = MD_CONFIG;
            changed = 1;
         } else {
            if (status.work.mode != MD_COUNTDOWN) {
               status.work.countDown = status.work.counter;
               status.work.mode = MD_COUNTDOWN;
               PORT_UV_SWITCH  |= 1 << PIN_UV_SWITCH;
               changed = 1;
            }
         }
      }

      /*
       * button 2 is "toggle light"
       */
      if (status.btT2 > BUTTON_SAMPLING) {
         status.btT2 = 0;
         if (status.work.mode != MD_COUNTDOWN) {
            if (status.work.mode == MD_LIGHT) {
               status.work.mode = MD_CONFIG;
               PORT_LIGHT_SWITCH &= (uint8_t) ~(1 << PIN_LIGHT_SWITCH);
            } else {
               status.work.mode = MD_LIGHT;
               PORT_LIGHT_SWITCH |= 1 << PIN_LIGHT_SWITCH;
            }
            changed = 1;
         }
      }

      /*
       * button 3 decreases countdown value
       */
      if (status.btT3 > BUTTON_SAMPLING) {
         status.btT3 = 0;
         if (status.work.mode == MD_CONFIG || status.work.mode == MD_DONE) {
            status.work.mode = MD_CONFIG;
            status.work.counter--;
            if (status.work.counter < 0)
               status.work.counter = COUNTER_MAX;
            changed = 1;
         }
      }

      /*
       * button 4 increases countdown value
       */
      if (status.btT4 > BUTTON_SAMPLING) {
         status.btT4 = 0;
         if (status.work.mode == MD_CONFIG || status.work.mode == MD_DONE) {
            status.work.mode = MD_CONFIG;
            status.work.counter++;
            if (status.work.counter > COUNTER_MAX)
               status.work.counter = 0;
            changed = 1;
         }
      }

      if (status.time != status.otime) {
         status.otime = status.time;
#ifdef WANT_CLOCK         
         clock();
#endif
         switch (status.work.mode) {
         case MD_COUNTDOWN:
            if (--status.work.countDown < 0) { 
               status.work.mode = MD_DONE;
               PORT_UV_SWITCH &= (uint8_t) ~(1 << PIN_UV_SWITCH);
               changed = 1;
            }                                              // fall through
         case MD_CONFIG:
#ifndef HAVE_LCD
            if (changed)
               sdGenDisplay();
#endif            
            break;

         case MD_DONE:
            status.beep = 20;
            break;
         }
#ifdef HAVE_LCD         
         lcdDisplay();
#endif         
      }

#ifdef HAVE_LCD
      if (changed) lcdDisplay();
#else      
      if (step ^= 1) spiWrite((uint8_t *)&status.work.display.step0, 3, SYNC_ONCE);
      else           spiWrite((uint8_t *)&status.work.display.step1, 3, SYNC_ONCE);
#endif
   }
}
